@@ -20,6 +20,7 @@
 #include "lvm-string.h"
 #include "toolcontext.h"
 #include "lv_alloc.h"
+#include "pv_alloc.h"
 #include "display.h"
 #include "segtype.h"
 
@@ -36,7 +37,7 @@ static void _get_extents(struct lv_segment *seg)
 		if (seg->area[s].type != AREA_PV)
 			continue;
 
-		pv = seg->area[s].u.pv.pv;
+		pv = seg->area[s].u.pv.pvseg->pv;
 		count = seg->area_len;
 		pv->pe_alloc_count += count;
 	}
@@ -51,7 +52,7 @@ static void _put_extents(struct lv_segment *seg)
 		if (seg->area[s].type != AREA_PV)
 			continue;
 
-		pv = seg->area[s].u.pv.pv;
+		pv = seg->area[s].u.pv.pvseg->pv;
 
 		if (pv) {
 			count = seg->area_len;
@@ -61,22 +62,73 @@ static void _put_extents(struct lv_segment *seg)
 	}
 }
 
-struct lv_segment *alloc_lv_segment(struct pool *mem, uint32_t num_areas)
+struct lv_segment *alloc_lv_segment(struct pool *mem,
+				    struct segment_type *segtype,
+				    struct logical_volume *lv,
+				    uint32_t le, uint32_t len,
+				    uint32_t status,
+				    uint32_t stripe_size,
+				    uint32_t area_count,
+				    uint32_t area_len,
+				    uint32_t chunk_size,
+				    uint32_t extents_copied)
 {
 	struct lv_segment *seg;
-	uint32_t len = sizeof(*seg) + (num_areas * sizeof(seg->area[0]));
+	uint32_t sz = sizeof(*seg) + (area_count * sizeof(seg->area[0]));
 
-	if (!(seg = pool_zalloc(mem, len))) {
+	if (!(seg = pool_zalloc(mem, sz))) {
 		stack;
 		return NULL;
 	}
 
-	seg->area_count = num_areas;
+	seg->segtype = segtype;
+	seg->lv = lv;
+	seg->le = le;
+	seg->len = len;
+	seg->status = status;
+	seg->stripe_size = stripe_size;
+	seg->area_count = area_count;
+	seg->area_len = area_len;
+	seg->chunk_size = chunk_size;
+	seg->extents_copied = extents_copied;
 	list_init(&seg->tags);
 
 	return seg;
 }
 
+int set_lv_segment_area_pv(struct lv_segment *seg, uint32_t area_num,
+			   struct physical_volume *pv, uint32_t pe)
+{
+	seg->area[area_num].type = AREA_PV;
+
+	if (!(seg->area[area_num].u.pv.pvseg =
+	      assign_peg_to_lvseg(pv, pe, seg->area_len, seg, area_num))) {
+		stack;
+		return 0;
+	}
+
+	return 1;
+}
+
+void set_lv_segment_area_lv(struct lv_segment *seg, uint32_t area_num,
+			    struct logical_volume *lv, uint32_t le)
+{
+	seg->area[area_num].type = AREA_LV;
+	seg->area[area_num].u.lv.lv = lv;
+	seg->area[area_num].u.lv.le = le;
+}
+
+static void _shrink_lv_segment(struct lv_segment *seg)
+{
+	uint32_t s;
+
+	for (s = 0; s < seg->area_count; s++) {
+		if (seg->area[s].type != AREA_PV)
+			continue;
+		release_pv_segment(seg->area[s].u.pv.pvseg, seg->area_len);
+	}
+}
+
 static int _alloc_parallel_area(struct logical_volume *lv, uint32_t area_count,
 				uint32_t stripe_size,
 				struct segment_type *segtype,
@@ -98,24 +150,21 @@ static int _alloc_parallel_area(struct logical_volume *lv, uint32_t area_count,
 	if (smallest < area_len)
 		area_len = smallest;
 
-	if (!(seg = alloc_lv_segment(lv->vg->cmd->mem, area_count))) {
-		log_err("Couldn't allocate new parallel segment.");
+	if (!(seg = alloc_lv_segment(lv->vg->cmd->mem, segtype, lv, *ix,
+				     area_len * (striped ? area_count : 1),
+				     0u, stripe_size, area_count, area_len,
+				     0u, 0u))) {
+		log_error("Couldn't allocate new parallel segment.");
 		return 0;
 	}
 
-	seg->lv = lv;
-	seg->segtype = segtype;
-	seg->le = *ix;
-	seg->len = area_len * (striped ? area_count : 1);
-	seg->area_len = area_len;
-	seg->stripe_size = stripe_size;
-	seg->extents_copied = 0u;
-
 	for (s = 0; s < area_count; s++) {
 		struct pv_area *pva = areas[s];
-		seg->area[s].type = AREA_PV;
-		seg->area[s].u.pv.pv = pva->map->pvl->pv;
-		seg->area[s].u.pv.pe = pva->start;
+		if (!set_lv_segment_area_pv(seg, s, pva->map->pvl->pv,
+					    pva->start)) {
+			stack;
+			return 0;
+		}
 		consume_pv_area(pva, area_len);
 	}
 
@@ -221,34 +270,34 @@ static int _alloc_linear_area(struct logical_volume *lv, uint32_t *ix,
 {
 	uint32_t count, remaining;
 	struct lv_segment *seg;
+	struct segment_type *segtype;
 
 	count = pva->count;
 	remaining = lv->le_count - *ix;
 	if (count > remaining)
 		count = remaining;
 
-	if (!(seg = alloc_lv_segment(lv->vg->cmd->mem, 1))) {
-		log_err("Couldn't allocate new stripe segment.");
+	if (!(segtype = get_segtype_from_string(lv->vg->cmd, "striped"))) {
+		stack;
 		return 0;
 	}
 
-	seg->lv = lv;
-	if (!(seg->segtype = get_segtype_from_string(lv->vg->cmd, "striped"))) {
+	if (!(seg = alloc_lv_segment(lv->vg->cmd->mem, segtype, lv, *ix,
+				     count, 0, 0, 1, count, 0, 0))) {
+		log_error("Couldn't allocate new stripe segment.");
+		return 0;
+	}
+
+	if (!set_lv_segment_area_pv(seg, 0, map->pvl->pv, pva->start)) {
 		stack;
 		return 0;
 	}
-	seg->le = *ix;
-	seg->len = count;
-	seg->area_len = count;
-	seg->stripe_size = 0;
-	seg->area[0].type = AREA_PV;
-	seg->area[0].u.pv.pv = map->pvl->pv;
-	seg->area[0].u.pv.pe = pva->start;
 
 	list_add(&lv->segments, &seg->list);
 
-	consume_pv_area(pva, count);
-	*ix += count;
+	consume_pv_area(pva, seg->len);
+	*ix += seg->len;
+
 	return 1;
 }
 
@@ -266,30 +315,24 @@ static int _alloc_mirrored_area(struct logical_volume *lv, uint32_t *ix,
 	if (count > remaining)
 		count = remaining;
 
-	if (!(seg = alloc_lv_segment(lv->vg->cmd->mem, 2))) {
+	if (!(seg = alloc_lv_segment(lv->vg->cmd->mem, segtype, lv, *ix,
+				     count, 0, 0, 2, count, 0, 0))) {
 		log_err("Couldn't allocate new mirrored segment.");
 		return 0;
 	}
 
-	seg->lv = lv;
-	seg->segtype = segtype;
-	seg->le = *ix;
-	seg->status = 0u;
-	seg->len = count;
-	seg->area_len = count;
-	seg->stripe_size = 0;
-	seg->extents_copied = 0u;
 	/* FIXME Remove AREA_PV restriction here? */
-	seg->area[0].type = AREA_PV;
-	seg->area[0].u.pv.pv = mirrored_pv;
-	seg->area[0].u.pv.pe = mirrored_pe;
-	seg->area[1].type = AREA_PV;
-	seg->area[1].u.pv.pv = map->pvl->pv;
-	seg->area[1].u.pv.pe = pva->start;
+	if (!set_lv_segment_area_pv(seg, 0, mirrored_pv, mirrored_pe) ||
+	    !set_lv_segment_area_pv(seg, 1, map->pvl->pv, pva->start)) {
+		stack;
+		return 0;
+	}
+
 	list_add(&lv->segments, &seg->list);
 
-	consume_pv_area(pva, count);
-	*ix += count;
+	consume_pv_area(pva, seg->len);
+	*ix += seg->len;
+
 	return 1;
 }
 
@@ -421,25 +464,44 @@ static int _alloc_virtual(struct logical_volume *lv,
 {
 	struct lv_segment *seg;
 
-	if (!(seg = alloc_lv_segment(lv->vg->cmd->mem, 0))) {
-		log_err("Couldn't allocate new zero segment.");
+	if (!(seg = alloc_lv_segment(lv->vg->cmd->mem, segtype, lv, allocated,
+				     lv->le_count - allocated, 0, 0, 0,
+				     lv->le_count - allocated, 0, 0))) {
+		log_error("Couldn't allocate new zero segment.");
 		return 0;
 	}
 
-	seg->lv = lv;
-	seg->segtype = segtype;
-	seg->status = 0u;
-	seg->le = allocated;
-	seg->len = lv->le_count - allocated;
-	seg->area_len = seg->len;
-	seg->stripe_size = 0;
-	seg->extents_copied = 0u;
 	list_add(&lv->segments, &seg->list);
 	lv->status |= VIRTUAL;
 
 	return 1;
 }
 
+struct lv_segment *alloc_snapshot_seg(struct logical_volume *lv,
+				      uint32_t allocated)
+{
+	struct lv_segment *seg;
+	struct segment_type *segtype;
+
+	segtype = get_segtype_from_string(lv->vg->cmd, "snapshot");
+	if (!segtype) {
+		log_error("Failed to find snapshot segtype");
+		return NULL;
+	}
+
+	if (!(seg = alloc_lv_segment(lv->vg->cmd->mem, segtype, lv, allocated,
+				     lv->le_count - allocated, 0, 0, 0,
+				     lv->le_count - allocated, 0, 0))) {
+		log_error("Couldn't allocate new snapshot segment.");
+		return NULL;
+	}
+
+	list_add(&lv->segments, &seg->list);
+	lv->status |= VIRTUAL;
+
+	return seg;
+}
+
 /*
  * Chooses a correct allocation policy.
  */
@@ -543,8 +605,10 @@ static char *_generate_lv_name(struct volume_group *vg, const char *format,
 struct logical_volume *lv_create_empty(struct format_instance *fi,
 				       const char *name,
 				       const char *name_format,
+				       union lvid *lvid,
 				       uint32_t status,
 				       alloc_policy_t alloc,
+				       int import,
 				       struct volume_group *vg)
 {
 	struct cmd_context *cmd = vg->cmd;
@@ -565,7 +629,8 @@ struct logical_volume *lv_create_empty(struct format_instance *fi,
 		return NULL;
 	}
 
-	log_verbose("Creating logical volume %s", name);
+	if (!import)
+		log_verbose("Creating logical volume %s", name);
 
 	if (!(ll = pool_zalloc(cmd->mem, sizeof(*ll))) ||
 	    !(ll->lv = pool_zalloc(cmd->mem, sizeof(*ll->lv)))) {
@@ -592,9 +657,14 @@ struct logical_volume *lv_create_empty(struct format_instance *fi,
 	lv->minor = -1;
 	lv->size = UINT64_C(0);
 	lv->le_count = 0;
+	lv->snapshot = NULL;
+	list_init(&lv->snapshot_segs);
 	list_init(&lv->segments);
 	list_init(&lv->tags);
 
+	if (lvid)
+		lv->lvid = *lvid;
+
 	if (fi->fmt->ops->lv_setup && !fi->fmt->ops->lv_setup(fi, lv)) {
 		stack;
 		if (ll)
@@ -602,7 +672,9 @@ struct logical_volume *lv_create_empty(struct format_instance *fi,
 		return NULL;
 	}
 
-	vg->lv_count++;
+	if (!import)
+		vg->lv_count++;
+
 	list_add(&vg->lvs, &ll->list);
 
 	return lv;
@@ -672,6 +744,8 @@ int lv_reduce(struct format_instance *fi,
 			/* remove this segment completely */
 			count -= seg->len;
 			_put_extents(seg);
+			seg->area_len = 0;
+			_shrink_lv_segment(seg);
 			list_del(segh);
 		} else {
 			/* reduce this segment */
@@ -687,6 +761,7 @@ int lv_reduce(struct format_instance *fi,
 			}
 			seg->area_len -=
 			    count / (striped ? seg->area_count : 1);
+			_shrink_lv_segment(seg);
 			_get_extents(seg);
 			count = 0;
 		}
@@ -706,7 +781,7 @@ int lv_reduce(struct format_instance *fi,
 
 int lv_remove(struct volume_group *vg, struct logical_volume *lv)
 {
-	struct list *segh;
+	struct lv_segment *seg;
 	struct lv_list *lvl;
 
 	/* find the lv list */
@@ -716,8 +791,8 @@ int lv_remove(struct volume_group *vg, struct logical_volume *lv)
 	}
 
 	/* iterate through the lv's segments freeing off the pe's */
-	list_iterate(segh, &lv->segments)
-	    _put_extents(list_item(segh, struct lv_segment));
+	list_iterate_items(seg, &lv->segments)
+	    _put_extents(seg);
 
 	vg->lv_count--;
 	vg->free_count += lv->le_count;
